#include "simulat.h"  /* contains function prototypes and external variable declarations */
#include "nrutil.h"
#include "H2O.h"		/* contains water related function prototypes and ext. var. dec */
#include <math.h>
#include <stdio.h>

extern int NN,NL;

void GET_H2O_PROPS(float *spech,float *mass,float *vol,float *press,float *temp,float *c1,
		float *c2,float *enth,float *rho,float *voi,float *qual,float *G1temp,float *G2temp,
		float adj,int ISAT,int nmax)
{
	float T,P;
	float ERRTOL=0.0001;
	float Mg,Mf;
	
	int ITYPE=2,IFLAG=-1;
	int i;
	
	char ERRMSG[40];
	
	for(i=1;i<=NN;i++)
	{
		P=press[i];
		T=temp[i];
		VOL=vol[i];
	
		if(ISAT!=6)
			RHO=rho[i];
			
		SPECH=spech[i];
				
		IFLAG=PROP(&P,&T,ISAT,ITYPE,nmax,ERRTOL,ERRMSG,adj);
				
		if(IFLAG==1)
		{
			G1temp[i]=0.0;
			G2temp[i]=0.0;
		}
		else
		{
			G1temp[i]=G1T;
			G2temp[i]=G2T;
		}
		
		if(ISAT==6)
		{
			mass[i]=TOTM;
			rho[i]=TOTM/vol[i];
			enth[i]=TOTH;
		}
		
		temp[i]=T;
		
		CLIPP(&P);
			
		press[i]=P;
		
		voi[i]=VOID;
		qual[i]=X;
		
		Mg=X*TOTM;
		Mf=(1.0-X)*TOTM;
		
		c1[i]=F1/(Mg*F4+Mf*F5)*1.0E6;
		c2[i]=F2/(Mg*F4+Mf*F5)*1.0E6;
	}
}

void LCOEF(float *k,float *w,float *dia,float *area,float *len,float *loss,float *den,
			float *rho,int *iup,float *qual,float *press)
{
	float Relo,friclo,viscf;
	int i;
	
	for(i=1;i<=NL;i++)
	{
		den[i]=rho[iup[i]];
		viscf=VISCF(press[iup[i]])*1.0e-6;

		/* calculate liquid only Reynolds number */
		
		Relo=fabs((w[i]*dia[i]/(area[i]*viscf)));
		
		/* calculate liquid only loss k */
		
		if(Relo!=0.0)
		{
			if(Relo<=1500)
				friclo=64.0/Relo;
			else
				friclo=0.0056+0.5/pow(Relo,0.32);
		
			k[i]=(friclo*len[i]/dia[i]+loss[i])/(2*den[i]*area[i]*len[i]);
		}
		else			
			k[i]=0.0;
			
		/* if flow is two-phase multiply by HEM two-phase friction multiplier */
		
		if(qual[iup[i]]>0.00001)
			k[i]=k[i]*(1+qual[iup[i]]*(VG(press[iup[i]])/VF(press[iup[i]])-1))
		  		*pow((1+qual[iup[i]]*(viscf/(VISCG(press[iup[i]])*1.0e-6)-1)),-0.25);
		  
	}
}

void PUMPHEAD(float *dppump)
{
	/* pumps do not presently contain a two-phase correction factor */
	
	dppump[9]=2.4338e6;
}

void HEATADDED(float *heat)
{
}

float TSTEP(float dt,float t,float *dtlast,float dtmin,float dtmax,float tfin,
			float *tprint,float terr,float pscale,float *dpdt)
{
	float dtp;
	int i;
	
	*dtlast=dt;
	
	dt=dtmax;
	
	if(dtmin!=dtmax)
	{
		for(i=1;i<=NN;i++)
		{
			if(fabs(dpdt[i])>=0.00001)		dtp=fabs(terr*pscale/dpdt[i]);
			if(dtp<dt)	 dt=dtp;
		}
	}
	
	if(dt<=dtmin)	dt=dtmin;

	if(t+dt>=tfin)
	{
		dt=tfin-t;
		*tprint=tfin;
	}
	
	if(dt>=*tprint-t)	dt=*tprint-t;
	
	return(dt);
}
